@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -192,7 +192,7 @@ class Node(Element, Observable):
 
     def _setLabelColor(self, color):
         '''Set label color attribute to Node'''
-        self.__labelColor = QColor(color) if color else None
+        self.__labelColor = color
         self._propagateEffectiveLabelColor(color)
 
     @property
@@ -201,17 +201,19 @@ class Node(Element, Observable):
 
     def _propagateEffectiveLabelColor(self, color):
         '''Set label color attribute to Node according inheritance policy'''
-        if self.__effectiveLabelColor == color: return
+        if equal(self.__effectiveLabelColor, color): return
         if self.__labelColor:
             self.__effectiveLabelColor = self.__labelColor
         else:
             self.__effectiveLabelColor = self.__parent.__effectiveLabelColor if self.__parent else color
+        if self.__effectiveLabelColor is not None:
+            self.__effectiveLabelColor = QColor(self.__effectiveLabelColor)
         for child in self.__children:
             child._propagateEffectiveLabelColor(self.__effectiveLabelColor)
 
     def _setLabelFont(self, font):
         '''Set label font attribute to Node'''
-        self.__labelFont = QFont(font) if font else None
+        self.__labelFont = font
         self._propagateEffectiveLabelFont(font)
 
     @property
@@ -220,18 +222,20 @@ class Node(Element, Observable):
 
     def _propagateEffectiveLabelFont(self, font):
         '''Set label font attribute to Node according inheritance policy'''
-        if self.__effectiveLabelFont == font: return
+        if equal(self.__effectiveLabelFont, font): return
         if self.__labelFont:
             self.__effectiveLabelFont = self.__labelFont
         else:
             self.__effectiveLabelFont = self.__parent.__effectiveLabelFont if self.__parent else font
+        if self.__effectiveLabelFont is not None:
+            self.__effectiveLabelFont = QFont(self.__effectiveLabelFont)
         self.__labelFontMetrics = QFontMetricsF(self.effectiveLabelFont)
         for child in self.__children:
             child._propagateEffectiveLabelFont(self.__effectiveLabelFont)
 
     def _setEdgeColor(self, color):
         '''Set edge color attribute to Node'''
-        self.__edgeColor = QColor(color) if color else None
+        self.__edgeColor = color
         self._propagateEffectiveEdgeColor(color)
 
     @property
@@ -240,11 +244,13 @@ class Node(Element, Observable):
 
     def _propagateEffectiveEdgeColor(self, color):
         '''Set edge color attribute to Node according inheritance policy'''
-        if self.__effectiveEdgeColor == color: return
+        if equal(self.__effectiveEdgeColor, color): return
         if self.__edgeColor:
             self.__effectiveEdgeColor = self.__edgeColor
         else:
             self.__effectiveEdgeColor = self.__parent.__effectiveEdgeColor if self.__parent else color
+        if self.__effectiveEdgeColor is not None:
+            self.__effectiveEdgeColor = QColor(self.__effectiveEdgeColor)
         for child in self.__children:
             child._propagateEffectiveEdgeColor(self.__effectiveEdgeColor)
 
@@ -289,7 +295,7 @@ class Node(Element, Observable):
 
     def _setFont(self, font):
         '''Set font value of Node text'''
-        self.__font = QFont(font) if font else None
+        self.__font = font
         self._propagateEffectiveFont(font)
 
     @property
@@ -298,18 +304,20 @@ class Node(Element, Observable):
 
     def _propagateEffectiveFont(self, font):
         '''Set font value of Node text according inheritance policy'''
-        if self.__effectiveFont == font: return
+        if equal(self.__effectiveFont, font): return
         if self.__font:
             self.__effectiveFont = self.__font
         else:
             self.__effectiveFont = self.__parent.__effectiveFont if self.__parent else font
+        if self.__effectiveFont is not None:
+            self.__effectiveFont = QFont(self.__effectiveFont)
         self.__richcontent.setDefaultFont(self.effectiveFont)
         for child in self.__children:
             child._propagateEffectiveFont(self.__effectiveFont)
 
     def _setColor(self, color):
         '''Set color value of Node text content'''
-        self.__color = QColor(color) if color else None
+        self.__color = color
         self._propagateEffectiveColor(color)
 
     @property
@@ -318,15 +326,38 @@ class Node(Element, Observable):
 
     def _propagateEffectiveColor(self, color):
         '''Set color value of Node text content according inheritance policy'''
-        if self.__effectiveColor == color: return
+        if equal(self.__effectiveColor, color): return
         if self.__color:
             self.__effectiveColor = self.__color
         else:
             self.__effectiveColor = self.__parent.__effectiveColor if self.__parent else color
+        if self.__effectiveColor is not None:
+            self.__effectiveColor = QColor(self.__effectiveColor)
         self.__colorPalette.setColor(QPalette.Text, self.effectiveColor)
         for child in self.__children:
             child._propagateEffectiveColor(self.__effectiveColor)
 
+    def _setBgColor(self, color):
+        '''Set background color value of Node'''
+        self.__background_color = color
+        self._propagateEffectiveBgColor(color)
+
+    @property
+    def effectiveBgColor(self):
+        return self.__effectiveBgColor if self.__effectiveBgColor else settings.get('defaultNodeBackgroundColor')
+
+    def _propagateEffectiveBgColor(self, color):
+        '''Set background color value of Node according inheritance policy'''
+        if equal(self.__effectiveBgColor, color): return
+        if self.__background_color:
+            self.__effectiveBgColor = self.__background_color
+        else:
+            self.__effectiveBgColor = self.__parent.__effectiveBgColor if self.__parent else color
+        if self.__effectiveBgColor is not None:
+            self.__effectiveBgColor = QColor(self.__effectiveBgColor)
+        for child in self.__children:
+            child._propagateEffectiveBgColor(self.__effectiveBgColor)
+
     def updateProperties(self):
         '''Update cached properties of the node'''
         self.__richcontent.setDefaultFont(self.effectiveFont)
@@ -365,6 +396,7 @@ class Node(Element, Observable):
         self.__font = None
         self.__effectiveFont = None
         self.__effectiveColor = None
+        self.__effectiveBgColor = None
         self.__icons = []
         self.__nodeAttribute = None
         self.__children = []
@@ -400,19 +432,17 @@ class Node(Element, Observable):
 #            # emit only one signal with max level of change
 #            self.notifyObservers(self, type = _Notifier.__SIGNALS[self.__maxNotifyLevel])
 
-    __NODE_BOUNDS_ATTRIBUTES = Set(('hgap', 'vgap', 'vshift', 'cloud', 'folded', 'icons', 'labelText'))
-    '''Properties which emit "nodeBoundsChanged" notification and not has a setter'''
-
-    __LOCAL_ATTRIBUTES = Set(('background_color', 'link'))
+    __LOCAL_ATTRIBUTES = Set(('cloud', 'folded', 'icons', 'labelText', 'link'))
     '''Properties which emit "localeAttributeChanged" notification and not has a setter'''
 
-    __NONEVENT_ATTRIBUTES = Set(('created', 'id', 'modified', 'encrypted_content', 'nodeAttribute', 'note'))
+    __NONEVENT_ATTRIBUTES = Set(('created', 'id', 'modified', 'encrypted_content', 'nodeAttribute', 'note',
+                                 'hgap', 'vgap', 'vshift'))
     '''Properties which not emit any notifications and not has a setter'''
 
-    __LOCAL_CHANGE_LEVEL, __NODE_BOUNDS_CHANGE_LEVEL, __INHERITED_CHANGE_LEVEL = range(3)
+    __LOCAL_CHANGE_LEVEL, __INHERITED_CHANGE_LEVEL = range(2)
     '''Numerical constants of signal types'''
 
-    __SIGNALS = ('localAttributeChanged', 'nodeBoundsChanged', 'inheritedAttributeChanged')
+    __SIGNALS = ('localAttributeChanged', 'inheritedAttributeChanged')
     '''Main signals to notify about mind map changes'''
 
     def setAttributes(self, **attributes):
@@ -427,32 +457,35 @@ class Node(Element, Observable):
         for attrName, attrValue in attributes.items():
             if attrName == 'font':
                 self._setFont(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'color':
                 self._setColor(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__LOCAL_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
+            elif attrName == 'background_color':
+                self._setBgColor(attrValue)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'text':
                 self._setPlainText(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__LOCAL_CHANGE_LEVEL)
             elif attrName == 'html':
                 self._setHtml(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__LOCAL_CHANGE_LEVEL)
 #           FIXME: differ levels by type
             elif attrName == 'edgeWidth':
                 self._setEdgeWidth(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'edgeStyle':
                 self._setEdgeStyle(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'edgeColor':
                 self._setEdgeColor(attrValue)
                 maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'labelFont':
                 self._setLabelFont(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'labelColor':
                 self._setLabelColor(attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__LOCAL_CHANGE_LEVEL)
+                maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
             elif attrName == 'left':
                 self._setLeft(attrValue)
                 maxNotifyLevel = max(maxNotifyLevel, Node.__INHERITED_CHANGE_LEVEL)
@@ -462,9 +495,6 @@ class Node(Element, Observable):
             elif attrName in Node.__LOCAL_ATTRIBUTES:
                 self._setField(attrName, attrValue)
                 maxNotifyLevel = max(maxNotifyLevel, Node.__LOCAL_CHANGE_LEVEL)
-            elif attrName in Node.__NODE_BOUNDS_ATTRIBUTES:
-                self._setField(attrName, attrValue)
-                maxNotifyLevel = max(maxNotifyLevel, Node.__NODE_BOUNDS_CHANGE_LEVEL)
             elif attrName in Node.__NONEVENT_ATTRIBUTES:
                 self._setField(attrName, attrValue)
             else: print 'Unknown attribute name: ' + attrName
@@ -586,7 +616,7 @@ class Node(Element, Observable):
         if self.__parent is None or self.isAncestorOf(dest):
             return
         if self.__parent == dest:
-            if dest.children.index(self) < position:
+            if position - dest.children.index(self) > 1:
                 position -= 1
         self.__parent.__children.remove(self)
         self.__parent = dest
@@ -602,6 +632,7 @@ class Node(Element, Observable):
         self._propagateEffectiveLabelColor(dest.effectiveLabelColor)
         self._propagateEffectiveLabelFont(dest.effectiveLabelFont)
         self._propagateEffectiveColor(dest.effectiveColor)
+        self._propagateEffectiveBgColor(dest.effectiveBgColor)
         self._propagateEffectiveFont(dest.effectiveFont)
         self.notifyObservers(self, dest, type = 'nodeMoved')
 
@@ -652,7 +683,8 @@ class Node(Element, Observable):
         child.__effectiveEdgeWidth = self.__effectiveEdgeWidth
         child.__effectiveEdgeStyle = self.__effectiveEdgeStyle
         child.__effectiveLabelColor = self.__effectiveLabelColor
-        child.__effectiveLabelFont = self.__effectiveLabelFont
+        child.__effectiveBgColor = self.__effectiveBgColor
+        child._propagateEffectiveLabelFont(self.__effectiveLabelFont)
         child._propagateEffectiveColor(self.__effectiveColor)
         child._propagateEffectiveFont(self.__effectiveFont)
         return child
@@ -794,3 +826,13 @@ class Attribute(Element):
         @type attrValue: str    
         '''
         self.__allAttributes[attrName] = attrValue
+
+def equal(objectOne, objectTwo):
+    '''
+    Compare two objects for equality. Both objects can be None.
+    @return: True if objects are equal, False otherwise
+    @rtype: bool
+    '''
+    if objectOne is None and objectTwo is None: return True
+    if objectOne is None or objectTwo is None: return False
+    return objectOne == objectTwo
