@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -218,7 +218,7 @@ class DesktopAboutDialog(AboutDialog):
         about.setFrameStyle(QFrame.NoFrame)
         about.setOpenExternalLinks(True)
         about.setHtml(self.tr('''HiveMind - Collaborative mind map editor<br>
-            Copyright &copy; 2010 HiveMind developers<br><br>
+            Copyright &copy; 2010-2011 HiveMind developers<br><br>
             License: <a href="http://www.gnu.org/licenses/gpl-3.0.html">
             GNU General Public License Version 3</a><br><br>
             HiveMind development is supported by<br>
@@ -258,8 +258,10 @@ class DesktopAboutDialog(AboutDialog):
         thanks.setHtml(self.tr('''
             Nicu Buculei<br>
             &nbsp;&nbsp;<a href="http://nicubunu.ro">nicubunu.ro</a><br>
-            &nbsp;&nbsp;HiveMind logo<br>
-            '''))
+            &nbsp;&nbsp;HiveMind logo<br><br>
+            Mentalrey<br>
+            &nbsp;&nbsp;<a href="http://www.mentalrey.it/">mentalrey.it</a><br>
+            &nbsp;&nbsp;Notification icons<br>'''))
         self._tabWidget.addTab(thanks, '&Thanks To')
         self._name = QLabel('<h2>HiveMind</h2>')
         self._version = QLabel(self.tr('<h4>Version %s</h4>') % __version__.VERSION)
@@ -348,10 +350,14 @@ class DesktopSettingsDialog(SettingsDialog):
                 'defaultEdgeWidth', 'defaultEdgeColor'])
         edgeGroup = QGroupBox(self.tr('Edge && label settings'))
         edgeGroup.setLayout(self._edge.getLayout())
+        self._autosave = SettingsCreator(['autosaveEnabled', 'autosaveInterval'])
+        autosaveGroup = QGroupBox(self.tr('Autosave settings'))
+        autosaveGroup.setLayout(self._autosave.getLayout())
         mainLayout = QVBoxLayout()
         mainLayout.addWidget(generalGroup)
         mainLayout.addWidget(nodeGroup)
         mainLayout.addWidget(edgeGroup)
+        mainLayout.addWidget(autosaveGroup)
         mainLayout.addWidget(self._buttonBox)
         self.setLayout(mainLayout)
 
@@ -362,6 +368,7 @@ class DesktopSettingsDialog(SettingsDialog):
         self._newSettings.append(self._general.retrieve())
         self._newSettings.append(self._node.retrieve())
         self._newSettings.append(self._edge.retrieve())
+        self._newSettings.append(self._autosave.retrieve())
         SettingsDialog._accept(self)
 
 
@@ -372,3 +379,12 @@ class DesktopXMPPCredentialsDialog(XMPPCredentialsDialog):
     '''
     def __init__(self, client = False, parent = None):
         XMPPCredentialsDialog.__init__(self, client, parent)
+
+
+class DesktopAffiliationDialog(AffiliationDialog):
+    '''
+    Affiliation dialog for Desktop platform
+    @author: Oleg Kandaurov
+    '''
+    def __init__(self, parent = None):
+        AffiliationDialog.__init__(self, parent)
