@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -157,7 +157,8 @@ class EditNodeDialog(CommonDialog):
         if self._isHtml:
             if self._isHtmlSourceMode:
                 storeAttrs['html'] = self._htmlEditor.toPlainText()
-            storeAttrs['html'] = self._editor.toHtml()
+            else:
+                storeAttrs['html'] = self._editor.toHtml()
         else:
             storeAttrs['font'] = self.__font
             storeAttrs['text'] = self._editor.toPlainText()
@@ -349,14 +350,15 @@ class FontDialog(CommonDialog):
         if index == -1:
             self.__comboSize.setEditText(str(font.pointSize()))
         else: self.__comboSize.setCurrentIndex(index)
-        self.__sampleText = QLabel(self.tr('ABCDE abcde'), self)
-        self.__sampleText.setStyleSheet('QLabel {color : black; background : white}')
+        self.__sampleText = QLineEdit(self.tr('ABCDE abcde'))
+        self.__sampleText.setFrame(False)
         self.__sampleText.setFont(QFont(self.__comboFont.currentFont().family(), int(self.__comboSize.currentText())))
         self.__sampleText.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
         layout = QGridLayout()
         layout.addWidget(self.__comboFont, 0, 0, 1, 3)
         layout.addWidget(self.__comboSize, 0, 3)
-        layout.addWidget(self.__sampleText, 1, 0, 1, 4)
+        scalingWidget = ScalingWidget(self.__sampleText, 2.0)
+        layout.addWidget(scalingWidget, 1, 0, 1, 4)
         layout.addWidget(self._buttonBox, 2, 0, 1, 4)
         self.setLayout(layout)
         self.__comboFont.connect(SIGNAL('currentFontChanged(QFont)'), self._setLabelFont)
@@ -500,7 +502,8 @@ class EditLabelDialog(CommonDialog):
         '''
         if font is None:
             font = self.__node.effectiveLabelFont
-        font.setPointSize(int(self._comboSize.currentText()))
+        if self._comboSize.currentText() != '':
+            font.setPointSize(int(self._comboSize.currentText()))
         self._fontSizes(font)
         self._setSizeIndex(font)
         self._setEditorFont(font)
@@ -516,10 +519,8 @@ class EditLabelDialog(CommonDialog):
         @type index: int
         '''
         font = self._comboFont.currentFont()
-        if index == -1:
-            self._comboSize.addItem(str(self.__font.pointSize()))
-            return
-        font.setPointSize(int(self._comboSize.itemText(index)))
+        if index != -1:
+            font.setPointSize(int(self._comboSize.itemText(index)))
         self._setEditorFont(font)
         self._updateButtonsState(font)
         # don't save current formatting if parent button checked
@@ -886,6 +887,8 @@ class XMPPCredentialsDialog(CommonDialog):
         self._passwordLabel = QLabel(self.tr('Password:'))
         self._serviceBinder = LineEditBinder('serviceJid', settings.get('serviceJid'), mailRegexp)
         self._serviceLabel = QLabel(self.tr('Service JID:'))
+        self._readOnlyBinder = CheckBoxBinder('readOnly', settings.get('readOnly'))
+        self._readOnlyBinder.setText(self.tr('Read only'))
 
     def _initComponents(self):
         mainLayout = QVBoxLayout()
@@ -894,6 +897,8 @@ class XMPPCredentialsDialog(CommonDialog):
         formLayout.addRow(self._passwordLabel, self._passwordBinder)
         if self.__client:
             formLayout.addRow(self._serviceLabel, self._serviceBinder)
+        else:
+            formLayout.addRow(self._readOnlyBinder)
         mainLayout.addLayout(formLayout)
         mainLayout.addWidget(self._buttonBox)
         self.setLayout(mainLayout)
@@ -910,6 +915,8 @@ class XMPPCredentialsDialog(CommonDialog):
         self.__newAttributes['userJid'] = self._userBinder.retrieve().values().pop()
         if self.__client:
             self.__newAttributes['serviceJid'] = self._serviceBinder.retrieve().values().pop()
+        else:
+            self.__newAttributes['readOnly'] = self._readOnlyBinder.retrieve().values().pop()
         settings.set(**self.__newAttributes)
         self.__newAttributes['password'] = self._passwordBinder.retrieve().values().pop()
         QDialog.accept(self)
@@ -970,3 +977,133 @@ class SideSelectionDialog(CommonDialog):
         '''
         self.__leftSide = True if self.sender() == self.__left else False
         QDialog.accept(self)
+
+
+class AffiliationDialog(CommonDialog):
+    '''
+    Abstract class for affiliation dialog
+    @author: Oleg Kandaurov
+    '''
+
+    def __init__(self, parent = None):
+        CommonDialog.__init__(self, parent)
+        self.setWindowTitle(self.tr('Edit affiliations'))
+        self.__supportedStatuses = {'subscribed': self.tr('Subscribed'),
+                'unsubscribed': self.tr('Unsubscribed')}
+        self._createComponents()
+        self._initComponents()
+
+    def _createComponents(self):
+        '''
+        Create components
+        '''
+        self.__model = QStandardItemModel(0, 3)
+        self.__delegate = AffiliationDelegate()
+        self.__view = QTableView()
+        self.__model.setHorizontalHeaderLabels([self.tr('JID'),
+                self.tr('Affiliation'), self.tr('Status')])
+        self.__view.setModel(self.__model)
+        self.__view.setItemDelegateForColumn(1, self.__delegate)
+        self.__jidLabel = QLabel(self.tr('JID'))
+        self.__jidEdit = QLineEdit()
+        self.__affiliationLabel = QLabel(self.tr('Affiliation'))
+        self.__affiliationCombo = QComboBox()
+        for data, name in self.__delegate.supportedAffiliations.iteritems():
+            self.__affiliationCombo.addItem(name, data)
+        self.__additionButton = QPushButton(self.tr('Add participant'))
+        self.__additionButton.clicked.connect(self._addParticipant)
+
+    def _initComponents(self):
+        '''
+        Initialize components
+        '''
+        jidLayout = QVBoxLayout()
+        self.__jidLabel.setAlignment(Qt.AlignCenter)
+        jidLayout.addWidget(self.__jidLabel)
+        jidLayout.addWidget(self.__jidEdit)
+        affilationLayout = QVBoxLayout()
+        self.__affiliationLabel.setAlignment(Qt.AlignCenter)
+        affilationLayout.addWidget(self.__affiliationLabel)
+        affilationLayout.addWidget(self.__affiliationCombo)
+        buttonLayout = QVBoxLayout()
+        buttonLayout.addWidget(QLabel(''))
+        buttonLayout.addWidget(self.__additionButton)
+        additionLayout = QHBoxLayout()
+        additionLayout.setAlignment(Qt.AlignBottom)
+        additionLayout.addLayout(jidLayout)
+        additionLayout.addLayout(affilationLayout)
+        additionLayout.addLayout(buttonLayout)
+        mainLayout = QVBoxLayout()
+        mainLayout.addWidget(self.__view)
+        mainLayout.addLayout(additionLayout)
+        mainLayout.addWidget(self._buttonBox)
+        self.setLayout(mainLayout)
+
+    def _addParticipant(self):
+        '''
+        Retrieve info from components and create participant with given access
+        '''
+        jidRegexp = QRegExp('[A-Za-z0-9._%+-]+@(?:[A-Za-z0-9-]+\\.)+[A-Za-z]{2,6}')
+        if not jidRegexp.exactMatch(self.__jidEdit.text()):
+            QMessageBox.critical(self, self.tr('Incorrect JID!'),
+                    self.tr('Please, enter correct jabber id.'))
+            return
+        self.setAffiliation(self.__jidEdit.text(),
+                self.__affiliationCombo.itemData(self.__affiliationCombo.currentIndex()))
+        self.__jidEdit.clear()
+
+    def updateStatus(self, event, subscriber, node, optional):
+        '''
+        Update status of widgets according to received event
+        @param event: Type of event
+        @type event: str
+        @param subscriber: Subcriber that generates event
+        @type subscriber: str
+        @param node: Node from which event received
+        @type node: str
+        @param optional: Subscribers count
+        @type optional: int
+        '''
+        for row in range(0, self.__model.rowCount()):
+            if self.__model.index(row, 0).data() == subscriber:
+                self.__model.setData(self.__model.index(row, 2),
+                        self.__supportedStatuses[event])
+
+    def setAffiliation(self, jid, newAffiliation):
+        '''
+        Set new affiliation
+        @param jid: jid to set affiliation
+        @type jid: str
+        @param affiliation: Affiliation for jid
+        @type affiliation: str
+        '''
+        for row in range(0, self.__model.rowCount()):
+            if self.__model.index(row, 0).data() == jid:
+                self.__model.setData(self.__model.index(row, 1),
+                        newAffiliation)
+                return
+        self.__model.setRowCount(self.__model.rowCount() + 1)
+        self.__model.setData(self.__model.index(self.__model.rowCount() - 1, 2),
+                self.__supportedStatuses['subscribed'])
+        self.__model.setData(self.__model.index(self.__model.rowCount() - 1, 1),
+                newAffiliation)
+        self.__model.setData(self.__model.index(self.__model.rowCount() - 1, 0),
+                jid)
+
+    def getAffiliations(self):
+        '''
+        Extract affiliations from model
+        @return: Dictionary of jid and affiliation
+        @rtype: dict
+        '''
+        affiliations = {}
+        for row in range(0, self.__model.rowCount()):
+            affiliations[self.__model.index(row, 0).data()] = \
+                    self.__model.index(row, 1).data()
+        return affiliations
+
+    def _accept(self):
+        '''
+        Hide the dialog
+        '''
+        QDialog.accept(self)
