@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -43,7 +43,9 @@ class MindMapDelegate(QGraphicsScene):
     @authors: Andrew Golovchenko, Andrew Vasilev, Alexander Kulikov
     '''
 
-    readable('rootDelegate', 'mindMap')
+    readable('rootDelegate', 'mindMap', 'selectLeftNodeAction', 'selectRightNodeAction',
+             'selectUpNodeAction', 'selectDownNodeAction', 'selectRootNodeAction')
+    writable('currentLayerDepth')
 
     def _getCurrentNode(self):
         '''Get current node'''
@@ -74,10 +76,32 @@ class MindMapDelegate(QGraphicsScene):
         self.__signalHandlers = {'childNodeAdded' : self.addNodeDelegate,
                                  'childNodeRemoved' : self.removeNodeDelegate,
                                  'nodeMoved' : self.relocateNodeDelegate,
-                                 'nodeBoundsChanged' : self.nodeBoundsChanged,
-                                 'localAttributeChanged' : self.repaintNode,
-                                 'inheritedAttributeChanged' : self.update}
+                                 'localAttributeChanged' : self.nodeBoundsChanged,
+                                 'inheritedAttributeChanged' : self.inheritedAttributeChanged}
         self.setMap(MindMap())
+        self._createNavigateActions()
+
+    def createAction(self, text, handler, shortcut = None, icon = None):
+        '''
+        Create action
+        @param text: descriptive text for tool buttons
+        @type text: string
+        @param handler: slot
+        @type shortcut: string
+        @rtype: QAction
+        '''
+        action = QAction(text, self)
+        if shortcut: action.setShortcut(shortcut)
+        if icon: action.setIcon(QIcon.fromTheme(icon))
+        if handler: action.connect(SIGNAL('triggered()'), handler)
+        return action
+
+    def _createNavigateActions(self):
+        self.__selectLeftNodeAction = self.createAction(None, self._selectLeftNode, 'Left')
+        self.__selectRightNodeAction = self.createAction(None, self._selectRightNode, 'Right')
+        self.__selectUpNodeAction = self.createAction(None, self._selectUpNode, 'Up')
+        self.__selectDownNodeAction = self.createAction(None, self._selectDownNode, 'Down')
+        self.__selectRootNodeAction = self.createAction(None, self._selectRootNode, 'Escape')
 
     def addNodeDelegate(self, node):
         '''
@@ -88,6 +112,7 @@ class MindMapDelegate(QGraphicsScene):
         self._addDelegatesForNodeSubtree(node)
         self.positionNodes()
         self.currentNode = node
+        self.__currentLayerDepth = node.depth()
 
     def _addDelegatesForNodeSubtree(self, root):
         '''
@@ -113,6 +138,7 @@ class MindMapDelegate(QGraphicsScene):
             if self.__nodeToDelegate[node.children[0]].isVisible() != visible:
                 for child in node.children:
                     self.__nodeToDelegate[child].setVisible(visible)
+        self.__nodeToDelegate[node].updateNodeBounds(False)
         self.positionNodes(node)
 
     def _settingsChanged(self):
@@ -128,8 +154,7 @@ class MindMapDelegate(QGraphicsScene):
         @type sender: Observable
         '''
         notifyType = args.get('type')
-        args = () if notifyType == 'inheritedAttributeChanged' else values
-        self.__signalHandlers.get(notifyType)(*args)
+        self.__signalHandlers.get(notifyType)(*values)
 
     def removeNodeDelegate(self, node):
         '''
@@ -167,7 +192,9 @@ class MindMapDelegate(QGraphicsScene):
         roamingDelegate.setParentItem(parentDelegate)
         if dest.folded: roamingDelegate.hide()
         # node relocation require recalculate position of all nodes
+        roamingDelegate.updateNodeBounds()
         self.positionNodes()
+        self.__currentLayerDepth = node.depth()
 
     def setMap(self, mindMap):
         '''
@@ -187,6 +214,8 @@ class MindMapDelegate(QGraphicsScene):
         self.createNodeDelegates(self.__rootDelegate)
         self.positionNodes()
         self.currentNode = mindMap.root
+        self.__currentLayerDepth = 0
+        self.emit(SIGNAL('mapChanged'))
 
     _MAP_MARGIN_WIDTH = 300
     ''' Width of mind map margin '''
@@ -226,9 +255,10 @@ class MindMapDelegate(QGraphicsScene):
         self.update()
         self._updateSceneRectangle()
 
-    def repaintNode(self, node):
-        '''Repaint node bounding area'''
-        self.__nodeToDelegate[node].update()
+    def inheritedAttributeChanged(self, node):
+        '''Update node and it's subtree bounding rectangle'''
+        self.__nodeToDelegate[node].updateNodeBounds()
+        self.positionNodes()
 
     def getDelegate(self, node):
         '''
@@ -366,6 +396,91 @@ class MindMapDelegate(QGraphicsScene):
         '''
         return position.y() < item.textRectOnScene().center().y()
 
+    def _selectNextLeftRightNode(self, leftSubtree):
+        '''
+        Switch currentNode to left or to right node
+        @type leftSubtree: boolean
+        @param leftSubtree: select left node if true.
+        '''
+        nextNode = None
+        if self.currentNode.parent is None:
+            #node is root
+            if not self.currentNode.folded:
+                #node is not folded
+                #find first left(right) child
+                for node in self.currentNode.children:
+                    if node.left == leftSubtree:
+                        nextNode = node
+                        break
+        elif self.currentNode.left == leftSubtree:
+            #node is not root. Select first child
+            if self.currentNode.children and not self.currentNode.folded:
+                #node has children and is not folded
+                nextNode = self.currentNode.children[0]
+        else:
+            #select parent
+            nextNode = self.currentNode.parent
+        if nextNode:
+            self.currentNode = nextNode
+            self.__currentLayerDepth = nextNode.depth()
+
+    def _createNodesLayer(self, root, maxDepth, leftSubtree, selectedNodes):
+        '''
+        Find nodes with the depth equal to maxDepth in the both left and right subtrees
+        and put them to the list.
+        @type root: Node
+        @type maxDepth: int
+        @type leftSubtree: bool
+        @type selectNodes: list
+        '''
+        if not root.folded and maxDepth > 0:
+            for node in root.children:
+                if node.left == leftSubtree:
+                    self._createNodesLayer(node, maxDepth - 1, leftSubtree, selectedNodes)
+        if root.folded or maxDepth == 0 or len(root.children) == 0:
+            selectedNodes.append(root)
+
+    def _selectRootNode(self):
+        '''Select root node'''
+        self.currentNode = self.rootDelegate.node
+
+    def _selectLeftNode(self):
+        '''Select left node'''
+        self._selectNextLeftRightNode(True)
+
+    def _selectRightNode(self):
+        '''Select right node'''
+        self._selectNextLeftRightNode(False)
+
+    def _selectUpNode(self):
+        '''Select up node'''
+        nodes = []
+        self._createNodesLayer(self.rootDelegate.node,
+                self.__currentLayerDepth, self.currentNode.left, nodes)
+        for node in nodes:
+            if self.currentNode.isAncestorOf(node) or self.currentNode == node:
+                index = nodes.index(node)
+                if index > 0:
+                    self.currentNode = nodes[index - 1]
+                return
+
+    def _selectDownNode(self):
+        '''Select down node'''
+        nodes = []
+        self._createNodesLayer(self.rootDelegate.node,
+                self.__currentLayerDepth, self.currentNode.left, nodes)
+        for node in nodes:
+            if self.currentNode.isAncestorOf(node) or self.currentNode == node:
+                index = nodes.index(node)
+                if index + 1 < len(nodes):
+                    self.currentNode = nodes[index + 1]
+                return
+
+    def onItemClicked(self, rightButton, item, parent, childLocation):
+        if item is not None:
+            self.currentNode = item.node
+            self.__currentLayerDepth = item.node.depth()
+
 
 class NodeDelegate(QGraphicsItem):
     '''
@@ -401,7 +516,10 @@ class NodeDelegate(QGraphicsItem):
         self.__node = node
         self.__verticalSpace = 0
         self.__selected = False
+        self.__positionChanged = True
         QGraphicsItem.__init__(self, parent)
+        self.updateNodeBounds(False)
+        self._updateNodePosition()
 
     __NODE_VERTICAL_SPACING = 4
     '''Minimal vertical space required for node decorations and spacing'''
@@ -510,37 +628,45 @@ class NodeDelegate(QGraphicsItem):
             horizontalPosition = self._childHorizontalPosition(child)
             child.setPos(horizontalPosition, verticalPosition)
             baseVerticalPosition += child.verticalSpace
-#           FIXME: requires to extract in separate method,
-#                  but change the local variable 'baseVerticalPosition' ...
-            offsetPoint = child._delegateOffsetPoint()
-            if offsetPoint is not None:
-                if offsetPoint.y() > 0.0:
-                    child.setPos(child.pos() + offsetPoint)
-                    baseVerticalPosition += abs(offsetPoint.y())
-                else:
-                    child.setX(child.x() + offsetPoint.x())
-                    edgeLine = QLineF(self.mapFromItem(child, child._parentLinkPoint()),
-                                      self.mapFromItem(child, child.linkPoint))
-                    hPosition = horizontalPosition
-                    if child.node.left: hPosition += child.textBoundingRect().width()
-                    horizontalPositionLine = QLineF(hPosition, -50.0, hPosition, 50.0)
-                    intersectionType, intersectionPoint = horizontalPositionLine.intersect(edgeLine)
-                    if baseVerticalPosition < intersectionPoint.y():
-                        baseVerticalPosition += abs(intersectionPoint.y() - baseVerticalPosition) \
-                                + NodeDelegate.__NODE_VERTICAL_SPACING
-        # FIXME: remove this line with the refactoring of the label draw algorithm
         self._computeSoloVerticalSpace()
         self.__verticalSpace = max(self.__verticalSpace, baseVerticalPosition -
             initialVerticalPosition)
 
-    @property
-    def linkPoint(self):
+    def setPos(self, x, y):
         '''
-        Return connection point for joint link
+        Overloaded method. Set new position of the item
+        @type x: float
+        @type y: float
+        '''
+        if self.x() != x or self.y() != y:
+            QGraphicsItem.setPos(self, x, y)
+            self._updateNodePosition()
 
-        @rtype: QPointF
+    def _updateNodePosition(self):
+        '''Calculate new node position-aware properties'''
+        if self.parentItem() is not None:
+            self._calculateParentLinkPoint()
+        self._calculateBoundingRect()
+
+    def updateNodeBounds(self, propagate = True):
         '''
-        if self.__node.parent is None: return None
+        Calculate new bounds for the given node
+        @param propagate: propagate border calculation on children or not
+        @type propagate: bool
+        '''
+        self._calculateTextBoundingRect()
+        self._calculateLinkPoint()
+        if self.parentItem() is None:
+            self._updateNodePosition()
+        if propagate:
+            for delegate in self.childItems():
+                delegate.updateNodeBounds(self)
+
+    def _calculateLinkPoint(self):
+        '''Calculate connection point for joint link'''
+        if self.__node.parent is None:
+            self.__linkPoint = None
+            return
         linkPoint = QPointF()
         if self.__node.effectiveStyle == 'bubble':
             linkPoint.setY(self.textBoundingRect().height() / 2.0)
@@ -549,10 +675,21 @@ class NodeDelegate(QGraphicsItem):
                            self.__node.effectiveEdgeWidth / 2.0)
         if self.__node.left:
             linkPoint.setX(self.textBoundingRect().width())
-        return linkPoint
+        self.__linkPoint = linkPoint
+
+    @property
+    def linkPoint(self):
+        '''
+        Return connection point for joint link
+        @rtype: QPointF
+        '''
+        return None if self.__linkPoint is None else QPointF(self.__linkPoint)
 
     def paint(self, painter, option, widget):
-        '''Paint node content and its joint link'''
+        '''
+        Paint node content and its joint link
+        @type painter: QPainter
+        '''
         # calculate bounding rect (without joint link)
         boundRect = self.textBoundingRect()
         # join link color
@@ -562,8 +699,7 @@ class NodeDelegate(QGraphicsItem):
         # join link shape style
         edgeStyle = self.__node.effectiveEdgeStyle
         # node background color
-        bgColor = self.__node.background_color
-        if not bgColor: bgColor = settings.get('defaultNodeBackgroundColor')
+        bgColor = self.__node.effectiveBgColor
         if self.__selected:
             bgColor = settings.get('defaultSelectedNodeBackgroundColor')
         # TODO user color value from settings instead of Qt.darkGray
@@ -580,7 +716,7 @@ class NodeDelegate(QGraphicsItem):
             boundRect.setHeight(boundRect.height() - edgeWidth / 2.0)
             painter.fillRect(boundRect, bgColor)
             # some decorative setting
-            if edgeStyle.startswith('sharp'): edgeWidth = 0.5
+            if edgeStyle.startswith('sharp') and not self.__node.children: edgeWidth = 0.5
             painter.setPen(QPen(edgeColor, edgeWidth, Qt.SolidLine, Qt.RoundCap))
             painter.drawLine(boundRect.bottomLeft(), boundRect.bottomRight())
         # paint edge label
@@ -600,35 +736,19 @@ class NodeDelegate(QGraphicsItem):
         ''' Paint text label of node'''
         painter.setFont(self.__node.effectiveLabelFont)
         painter.setPen(self.__node.effectiveLabelColor)
-        if self.__node.effectiveEdgeStyle.endswith('bezier'):
-            labelPoint = self.linkPoint
-            if self.__node.left:
-                labelPoint.setX(labelPoint.x() + NodeDelegate.__LABEL_MARGIN)
-            else:
-                labelPoint.setX(labelPoint.x() - self.__node.labelSize.width() - NodeDelegate.__LABEL_MARGIN)
-            labelPoint.setY(labelPoint.y() - self.__node.effectiveEdgeWidth / 2.0 - 2.0)
-            painter.drawText(labelPoint, self.__node.labelText)
-            painter.setPen(QPen(self.__node.effectiveEdgeColor, self.__node.effectiveEdgeWidth,
-                    Qt.SolidLine, Qt.FlatCap))
-            linkPoint = self.linkPoint
-            painter.drawLine(linkPoint, QPointF(linkPoint.x() + self.linkPointOffset(), linkPoint.y()))
-        else: # linear or sharp linear
-            painter.save()
-            edgeLine = QLineF(self._parentLinkPoint(), self.linkPoint)
-            painter.translate(self.linkPoint)
-            labelPoint = QPointF()
-            if self.__node.left:
-                labelPoint.setX(self._nodeToLabelOffset())
-            else:
-                labelPoint.setX(-self._nodeToLabelOffset() - self.__node.labelSize.width())
-            labelPoint.setY(labelPoint.y() - self.__node.effectiveEdgeWidth / 2.0 - 2.0)
-            if edgeLine.dx() * edgeLine.dy() > 0.0:
-                painter.rotate(90.0 - math.fmod(edgeLine.angle(), 90.0))
-            else:
-                painter.rotate(-math.fmod(edgeLine.angle(), 90.0))
-            painter.setFont(self.__node.effectiveLabelFont)
-            painter.drawText(labelPoint, self.__node.labelText)
-            painter.restore()
+        labelPoint = self.linkPoint
+        if self.__node.left:
+            labelPoint.setX(labelPoint.x() + NodeDelegate.__LABEL_MARGIN)
+        else:
+            labelPoint.setX(labelPoint.x() - self.__node.labelSize.width() - NodeDelegate.__LABEL_MARGIN)
+        labelPoint.setY(labelPoint.y() - self.__node.effectiveEdgeWidth / 2.0 - 2.0)
+        painter.drawText(labelPoint, self.__node.labelText)
+        pen = QPen(self.__node.effectiveEdgeColor, self.__node.effectiveEdgeWidth, Qt.SolidLine, Qt.RoundCap)
+        if self.__node.effectiveEdgeStyle.startswith('sharp') and not self.__node.children:
+            pen.setWidthF(0.5)
+        painter.setPen(pen)
+        linkPoint = self.linkPoint
+        painter.drawLine(linkPoint, QPointF(linkPoint.x() + self.linkPointOffset(), linkPoint.y()))
 
     def _paintRoot(self, painter, boundRect):
         '''
@@ -717,24 +837,31 @@ class NodeDelegate(QGraphicsItem):
         self.__node.paint(painter)
         painter.restore()
 
-    def textBoundingRect(self):
+    def _calculateTextBoundingRect(self):
         '''Calculate bounding region only for node without link to the parent'''
         size = self.__node.textSize()
         if self.__node.icons:
             size.setWidth(size.width() + NodeDelegate.__TEXT_MARGIN
                     + len(self.__node.icons) * NodeDelegate.__ICON_SIZE)
             size.setHeight(max(size.height(), NodeDelegate.__ICON_SIZE))
-        if not self.parentItem(): return self._rootBoundingRect(size)
+        if not self.parentItem():
+            self.__textBoundingRect = self._rootBoundingRect(size)
+            return
         if self.__node.folded: size.setWidth(size.width() + NodeDelegate.__FOLD_MARK_SIZE)
         boundRect = QRectF(0.0, 0.0, size.width(), size.height())
         if self.__node.effectiveStyle == 'fork':
             boundRect.setHeight(boundRect.height() + self.__node.effectiveEdgeWidth / 2.0)
-        return boundRect
+        self.__textBoundingRect = boundRect
 
-    def boundingRect(self):
+    def textBoundingRect(self):
+        '''Text bounding rectangle of the node'''
+        return QRectF(self.__textBoundingRect)
+
+    def _calculateBoundingRect(self):
         '''Calculate bounding rectangle'''
         if self.parentItem() is None:
-            return self.textBoundingRect()
+            self.__boundingRect = self.textBoundingRect()
+            return
         linkPoint = self._parentLinkPoint()
         textBounds = self.textBoundingRect()
         left = min(linkPoint.x(), textBounds.x())
@@ -751,7 +878,11 @@ class NodeDelegate(QGraphicsItem):
         if self.__node.folded:
             width += NodeDelegate.__FOLD_MARK_SIZE
             if self.__node.effectiveStyle == 'fork': height += NodeDelegate.__FOLD_MARK_SIZE
-        return QRectF(left, top, width, height)
+        self.__boundingRect = QRectF(left, top, width, height)
+
+    def boundingRect(self):
+        '''Bounding rectangle of the node'''
+        return self.__boundingRect
 
     def textRectOnScene(self):
         '''
@@ -789,101 +920,6 @@ class NodeDelegate(QGraphicsItem):
         '''
         return self.mapRectToScene(self.boundingRect())
 
-    def _delegateOffsetPoint(self):
-        '''
-        @return: additional offset point of node delegate specified by label size.
-                 If no offset required, None value will be returned
-        @rtype: QPointF
-        '''
-        if self.__node.labelText is None or self.__node.effectiveEdgeStyle.endswith('bezier'):
-            return None
-        edgeLine = QLineF(self._parentLinkPoint(), self.linkPoint)
-        requiredLength = self.__node.labelSize.width() + self._nodeToLabelOffset() \
-                + self._neighborEdgeToLabelOffset()
-        if requiredLength > edgeLine.length():
-            if edgeLine.angle() < 180.0: # for top half of tree used only horizontal offset
-                horisontalOffset = math.sqrt(requiredLength * requiredLength \
-                        - edgeLine.dy() * edgeLine.dy())
-                horisontalOffset -= abs(edgeLine.dx())
-                if self.__node.left: horisontalOffset = -horisontalOffset
-                return QPointF(horisontalOffset, 0.0)
-            else: # offset along current edge line
-                lineDifference = abs(requiredLength - edgeLine.length())
-                edgeLine.setLength(lineDifference)
-                return QPointF(edgeLine.dx(), edgeLine.dy())
-        return None
-
-    def _nodeToLabelOffset(self):
-        '''
-        @return: offset specified of angle between edge and node area.
-
-        @rtype: float
-        '''
-        edgeLine = QLineF(self._parentLinkPoint(), self.linkPoint)
-        angle = edgeLine.angle()
-        if angle > 180.0: # only lower half of tree
-            offset = min(self.textBoundingRect().height() / 2.0, self.__node.labelSize.height())
-#            if edgeLine.dx() * edgeLine.dy() > 0.0:
-#                angle = math.radians(math.fmod(angle, 90.0))
-#            else:
-#                angle = math.radians(90.0 - math.fmod(angle, 90.0))
-#            oppositeCathet = self.__node.labelSize.height() + self.__node.effectiveEdgeWidth / 2.0
-#            offset = min(abs(oppositeCathet / math.tan(angle)), self.textBoundingRect().height() / 2.0)
-            return offset + NodeDelegate.__LABEL_MARGIN
-        return NodeDelegate.__LABEL_MARGIN
-
-    def _neighborEdgeToLabelOffset(self):
-        '''
-        @return: offset specified of angle between intersected neighbor edges.
-                 If offset is very large, label height-specified constant will be returned
-        @rtype: float
-        '''
-        edgeLine = QLineF(self._parentLinkPoint(), self.linkPoint)
-        topNeighborNode = self._nearestNeighborDelegate(True)
-        if topNeighborNode is None:
-            if self.__node.left: # use random line with 45.0 angle value
-                topNeighborEdge = QLineF(0.0, 0.0, 30.0, -30.0)
-            else: # use random line with 135.0 angle value
-                topNeighborEdge = QLineF(0.0, 0.0, -30.0, -30.0)
-        else: # use edge of top neighbor node
-#           TODO: may be use linkPoint + linkPoint offset
-            topNeighborEdge = QLineF(edgeLine.p1(),
-                    self.mapFromItem(topNeighborNode, topNeighborNode.linkPoint))
-        if self.__node.left:
-            angle = topNeighborEdge.angleTo(edgeLine)
-        else:
-            angle = edgeLine.angleTo(topNeighborEdge)
-        if angle < 90.0:
-            angle = math.radians(angle)
-            oppositeCathet = self.__node.labelSize.height() \
-                    + self.__node.effectiveEdgeWidth / 2.0
-            if topNeighborNode is not None:
-                oppositeCathet += topNeighborNode.node.effectiveEdgeWidth / 2.0
-            return min(abs(oppositeCathet / math.tan(angle)),
-                       self.__node.labelSize.height() * 2.0) + NodeDelegate.__LABEL_MARGIN
-        return NodeDelegate.__LABEL_MARGIN
-
-    def _nearestNeighborDelegate(self, upward = False):
-        '''
-        Search upper or under delegate in similar subtree
-
-        @param upward: direction of search: if True, upper delegate 
-                       will be returned, otherwise under delegate
-        @type upward: bool
-
-        @return: nearest delegate in a specified direction or None
-        @rtype: NodeDelegate
-        '''
-        children = self.__node.parent.children
-        step = -1 if upward else 1
-        stop = -1 if upward else len(children)
-        index = children.index(self.__node) + step
-        while index != stop:
-            if children[index].left is self.__node.left:
-                return self.scene().getDelegate(children[index])
-            index += step
-        return None
-
     def _labelOverhead(self):
         '''
         @return: difference between top points value of node (text bounding rect) and its label
@@ -899,27 +935,28 @@ class NodeDelegate(QGraphicsItem):
             labelTop -= textBounds.height() / 2.0
         return - labelTop if labelTop < 0.0 else 0.0
 
-    def _parentLinkPoint(self):
-        '''
-        Calculate connection point to parent node
-        
-        @return: connection point in current node coordinates
-        @rtype: QPointF
-        '''
+    def _calculateParentLinkPoint(self):
+        '''Calculate connection point to parent node'''
         parentLinkPoint = QPointF()
         parentRect = self.parentItem().textBoundingRect()
-        if not self.__node.left:
-            parentLinkPoint.setX(parentRect.width())
-        else:
-            parentLinkPoint.setX(0.0)
+        if not self.__node.left: parentLinkPoint.setX(parentRect.width())
         parentNode = self.parentItem().node
         # root node never has fork style and only root has no parent
         if parentNode.effectiveStyle == 'bubble' or parentNode.parent is None:
             parentLinkPoint.setY(parentRect.height() / 2.0)
         else:
-            parentRect.setHeight(parentRect.height() - self.__node.effectiveEdgeWidth / 2.0)
+            parentRect.setHeight(parentRect.height() - parentNode.effectiveEdgeWidth / 2.0)
             parentLinkPoint.setY(parentRect.height())
-        return self.mapFromParent(parentLinkPoint)
+        self.__parentLinkPoint = self.mapFromParent(parentLinkPoint)
+
+    def _parentLinkPoint(self):
+        '''
+        Get connection point to parent node
+
+        @return: connection point in current node coordinates
+        @rtype: QPointF
+        '''
+        return QPointF(self.__parentLinkPoint)
 
     def _paintEdge(self, painter, style):
         '''
@@ -954,7 +991,7 @@ class NodeDelegate(QGraphicsItem):
             p2.setX(curveRect.bottomLeft().x() + curveRect.width() / 2.0)
             curvePath.moveTo(startPoint.x(), startPoint.y() + self.__node.effectiveEdgeWidth / 2.0)
             curvePath.cubicTo(p1, p2, endPoint)
-            startPoint.setY(startPoint.y() - self.__node.effectiveEdgeWidth)
+            startPoint.setY(startPoint.y() - self.__node.effectiveEdgeWidth / 2.0)
             curvePath.cubicTo(p2, p1, startPoint)
             painter.drawPath(curvePath)
             painter.fillPath(curvePath, self.__node.effectiveEdgeColor)
@@ -982,7 +1019,7 @@ class NodeDelegate(QGraphicsItem):
         @rtype: float
         '''
         labelWidth = self.__node.labelSize.width()
-        if labelWidth == 0.0 or self.__node.effectiveEdgeStyle.endswith('linear'):
+        if labelWidth == 0.0:
             return 0.0
         offset = labelWidth + NodeDelegate.__LABEL_MARGIN * 2.0
         return offset if self.__node.left else - offset
