@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -49,6 +49,43 @@ class SearchingTrMixin(object):
             if clazz.__name__ == 'SearchingTrMixin': return text
         return text
 
+
+class ScalingWidget(QGraphicsView):
+    '''
+    Special widget for widgets scaling
+
+    @author: Alexander Kulikov
+    '''
+    def __init__(self, widget, scale, parent = None):
+        '''
+        Initialize widget
+
+        @type widget: QWidget
+        @type scale: int
+        @type parent: QWidget
+        '''
+        QGraphicsView.__init__(self, parent)
+        self.__widget = widget
+        self.__scene = QGraphicsScene()
+        self.setScene(self.__scene)
+        self.__scene.addWidget(widget)
+        self.__scale = scale
+        self.scale(self.__scale, self.__scale)
+        self.setRenderHints(QPainter.Antialiasing)
+
+    def resizeEvent(self, event):
+        '''
+        Resize event handler
+
+        @type event: QResizeEvent
+        '''
+        width = event.size().width() / self.__scale
+        height = event.size().height() / self.__scale
+        self.__scene.setSceneRect(QRect(0, 0, width, height))
+        self.__widget.resize(width, height)
+        QGraphicsView.resizeEvent(self, event)
+
+
 class SpecialTextEdit(QTextEdit):
     '''
     Special text edit widget which handles Enter as newline character insertion
@@ -199,6 +236,14 @@ class IconList(QGraphicsScene):
                            self.MAX_ICON_COUNT - self.__horizontalSpacing, \
                            self.__itemSize)
 
+    def _updatePosition(self):
+        '''Calculate new position for icon items'''
+        for i in xrange(0, len(self.__items)):
+            item = self.__items[i]
+            if isinstance(item, IconItem):
+                x = i * (self.__itemSize + self.__horizontalSpacing)
+                item.setPos(x, 0)
+
     def setItemSize(self, itemSize):
         '''
         Set item size of current items
@@ -209,16 +254,16 @@ class IconList(QGraphicsScene):
         for item in self.items():
             if isinstance(item, IconItem):
                 item.setSize(itemSize)
+        self._updatePosition()
 
     def setHorizontalSpacing(self, spacing):
+        '''
+        Set horizontal spacing beetwing icon items
+        @type spacing: int
+        '''
         self.__horizontalSpacing = spacing
         self._updateSize()
-        i = 0
-        for item in self.items():
-            if isinstance(item, IconItem):
-                left = i * (self.__itemSize + self.__horizontalSpacing)
-                i += 1
-                item.setPos(left, 0)
+        self._updatePosition()
 
     def setIconSize(self, iconSize):
         '''
@@ -341,78 +386,146 @@ class IconList(QGraphicsScene):
         group.start()
 
 
-class ImageButton(QAbstractButton):
+class ActionImageButton(QAbstractButton):
     '''
-    Command widget, only image is displayed to show the state of the button
+    Command widget that only shows the icon of the action and tightly assosiated with it.
     
     @author: Andrew Vasilev
     '''
 
-    def __init__(self, parent = None, baseImageName = ''):
+    __MAX_ICON_HEIGHT = 64
+    '''Maximum height and width of the icon'''
+
+    def __init__(self, action, parent = None, sizeHint = None):
         '''
-        @param parent: parent widget object
+        @param action: action to bound state of the button
+        @type action: QAction
+        @param parent: parent widget to paint button on
         @type parent: QWidget
-        
-        @param baseImageName: base name of an images to load
-        @type baseImageName: str
+        @param sizeHint: maximum height\width of the icon to draw
+        @type sizeHint: int
         '''
         QAbstractButton.__init__(self, parent)
-        self.__maxImageSize = QSize()
-        if baseImageName != '':
-            self.setImage(baseImageName)
+        self.__action = action
+        self._syncProperties()
+        self._connectToAction()
+        self._setupSizeHint(sizeHint)
 
-    __NORMAL_STATE_IMAGE_SUFFIX = '_normal'
-    '''Suffix for image to be displayed in normal mode'''
+    def _syncProperties(self):
+        '''Synchronize action properties with the button properties and update the view'''
+        self.setAutoRepeat(self.__action.autoRepeat())
+        self.setCheckable(self.__action.isCheckable())
+        self.setEnabled(self.__action.isEnabled())
+        self.update()
 
-    __SELECTED_STATE_IMAGE_SUFFIX = '_selected'
-    '''Suffix for image to be displayed in selected mode'''
+    def _connectToAction(self):
+        '''Create connections to the action state and property changes and enable acction
+        trigger on button click'''
+        self.__action.connect(SIGNAL('changed()'), self._syncProperties)
+        self.__action.connect(SIGNAL('toggled(bool)'), self.setChecked)
+        self.connect(SIGNAL('clicked()'), self.__action.trigger)
 
-    def setImage(self, baseImageName):
+    def _setupSizeHint(self, sizeHint):
         '''
-        @param baseImageName: base name of an images to load
-        @type baseImageName: str
+        Setup size hint attribute for the button. Cannot be more than 
+        ActionImageButton.__MAX_ICON_HEIGHT and the size of the icon.
+        @param sizeHint: proposed height/width of the button
+        @type sizeHint: int
         '''
-        self.__normalModeImage = resource.getImage(baseImageName +
-
-                                                   ImageButton.__NORMAL_STATE_IMAGE_SUFFIX)
-        self.__selectedModeImage = resource.getImage(baseImageName +
-                                                      ImageButton.__SELECTED_STATE_IMAGE_SUFFIX)
-        normalModeImageSize = self.__normalModeImage.size()
-        selectedModeImageSize = self.__selectedModeImage.size()
-        if normalModeImageSize.width() > selectedModeImageSize.width():
-            self.__maxImageSize = normalModeImageSize
-        else:
-            self.__maxImageSize = selectedModeImageSize
+        defaultSize = ActionImageButton.__MAX_ICON_HEIGHT if sizeHint is None else sizeHint
+        self.__sizeHint = QSize(defaultSize, defaultSize)
+        newSize = self.__action.icon().actualSize(self.__sizeHint)
+        if newSize.height() < self.__sizeHint.height():
+            self.__sizeHint = newSize
 
     def sizeHint(self):
         '''
-        @return: size of the button, maximum icon size
+        @return: recomendet size for the widget
         @rtype: QSize
         '''
-        return self.__maxImageSize
+        return self.__sizeHint
 
     def paintEvent(self, paintEvent):
-        '''Paint icon in corresponding state'''
+        '''
+        Paint action icon in the corresponding state
+        @param paintEvent: paint event representation
+        @type paintEvent: QPaintEvent 
+        '''
         painter = QPainter(self)
-        if self.isDown() or self.isChecked():
-            painter.drawPixmap(self.rect(), self.__selectedModeImage,
-                               self.__selectedModeImage.rect())
-        else:
-            painter.drawPixmap(self.rect(), self.__normalModeImage,
-                               self.__normalModeImage.rect())
+        if not self.__action.isEnabled(): mode = QIcon.Disabled
+        elif self.isDown() or self.__action.isChecked(): mode = QIcon.Selected
+        else: mode = QIcon.Normal
+        self.__action.icon().paint(painter, self.rect(), Qt.AlignCenter, mode)
 
-    def moveCenter(self, x, y):
+
+class MaemoConnectionStatusWidget(QWidget):
+    '''
+    Widget shows status of connection
+
+    @author: Oleg Kandaurov
+    '''
+    def __init__(self, parent, connectionAction, subscribeAction):
         '''
-        Move button center to specified coordinates
-        
-        @param x: x coordinate of button center
-        @type x: int
-        
-        @param y: y coordinate of button center
-        @type y: int
+        Initialize widget
+        @type parent: QWidget
+        @type connectionAction: StatusAction
+        @type subscribeAction: StatusAction
         '''
-        self.move(x - self.__maxImageSize.width() / 2.0,
-                  y - self.__maxImageSize.height() / 2.0)
+        QWidget.__init__(self, parent)
+        self.__connectionButton = ActionImageButton(connectionAction, parent, 48)
+        self.__subscribeButton = ActionImageButton(subscribeAction, parent, 48)
+        layout = QHBoxLayout()
+        layout.setContentsMargins(5, 5, 5, 5)
+        layout.addWidget(self.__connectionButton)
+        layout.addWidget(self.__subscribeButton)
+        self.setLayout(layout)
+
+
+class StatusAction(QAction):
+    '''
+    Action with support for statuses
+
+    @author: Oleg Kandaurov
+    '''
+    readable('image')
+
+    def __init__(self, text, type, status, parent = None):
+        '''
+        Constructs action
+        @param text: Text of the action
+        @type text: str
+        @param type: Type of the action
+        @type type: str
+        @param status: Initial status of the action
+        @type status: str
+        @param parent: Parent
+        @type parent: QObject
+        '''
+        QAction.__init__(self, text, parent)
+        SIGNAL('imageChanged(QImage)')
+        SIGNAL('statusChanged(str)')
+        self.__type = type
+        self.setStatus(status)
+        self.setEnabled(False)
+
+    def setStatus(self, status):
+        '''
+        Set status of the action and change icon according to status
+        '''
+        self.__status = status
+        self.__image = resource.getImage(str(self.__type) + '_' + str(self.__status))
+        icon = QIcon()
+        icon.addPixmap(self.__image, QIcon.Disabled)
+        self.setIcon(icon)
+        self.emit(SIGNAL('statusChanged'), self.__status)
+
+    def getStatus(self):
+        '''
+        Get status of the action
+        '''
+        return self.__status
+
+    status = property(getStatus, setStatus, None, 'Status of the Action')
 
 
 class MaemoZoomWidget(QWidget):
@@ -429,12 +542,12 @@ class MaemoZoomWidget(QWidget):
         @type zoomOutAction: QAction
         '''
         QWidget.__init__(self, parent)
-        self.__zoomInButton = ImageButton(self, 'zoom_in')
-        self.__zoomInButton.setAutoRepeat(True)
-        self.connect(self.__zoomInButton, SIGNAL('pressed()'), zoomInAction.trigger)
-        self.__zoomOutButton = ImageButton(self, 'zoom_out')
-        self.__zoomOutButton.setAutoRepeat(True)
-        self.connect(self.__zoomOutButton, SIGNAL('pressed()'), zoomOutAction.trigger)
+        zoomInAction.setIcon(resource.getIcon('zoom_in'))
+        zoomInAction.setAutoRepeat(True)
+        self.__zoomInButton = ActionImageButton(zoomInAction, parent)
+        zoomOutAction.setIcon(resource.getIcon('zoom_out'))
+        zoomOutAction.setAutoRepeat(True)
+        self.__zoomOutButton = ActionImageButton(zoomOutAction, parent)
         layout = QVBoxLayout()
         layout.setContentsMargins(5, 20, 5, 5)
         layout.addWidget(self.__zoomInButton)
@@ -512,19 +625,6 @@ class SettingsCreator(QObject):
         self.__settings = settings
         self.__mainLayout = QFormLayout()
         self.__component = dict()
-        self.__create = {'locale': self._createLocale,
-            'confirmDelete': self._createConfirmDelete,
-            'autoHideNodeMenuTimeout': self._createAutoHideNodeMenuTimeout,
-            'showNodeMenuAnimationDuration': self._createShowNodeMenuAnimationDuration,
-            'hideNodeMenuAnimationDuration': self._createHideNodeMenuAnimationDuration,
-            'defaultFont': self._createDefaultFont,
-            'defaultLabelFont': self._createDefaultLabelFont,
-            'defaultNodeTextColor': self._createDefaultNodeTextColor,
-            'defaultNodeBackgroundColor': self._createDefaultNodeBackgroundColor,
-            'defaultSelectedNodeBackgroundColor': self._createDefaultSelectedNodeBackgroundColor,
-            'defaultEdgeColor': self._createDefaultEdgeColor,
-            'defaultLabelColor': self._createDefaultLabelColor,
-            'defaultEdgeWidth': self._createDefaultEdgeWidth}
         self._createComponents()
 
     def getLayout(self):
@@ -535,7 +635,7 @@ class SettingsCreator(QObject):
         Create components
         '''
         for setting in self.__settings:
-            self.__create[setting]()
+            getattr(self, '_create%s' % setting)()
 
     def retrieve(self):
         '''
@@ -546,7 +646,7 @@ class SettingsCreator(QObject):
             newSettings.append(self.__component[setting].retrieve())
         return newSettings
 
-    def _createLocale(self):
+    def _createlocale(self):
         '''
         Create locale components
         '''
@@ -558,7 +658,7 @@ class SettingsCreator(QObject):
         self.__component['locale'] = binder
 
 
-    def _createConfirmDelete(self):
+    def _createconfirmDelete(self):
         '''
         Create confirmDelete components
         '''
@@ -567,7 +667,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(binder)
         self.__component['confirmDelete'] = binder
 
-    def _createAutoHideNodeMenuTimeout(self):
+    def _createautoHideNodeMenuTimeout(self):
         '''
         Create autoHideNodeMenuTimeout components
         '''
@@ -576,7 +676,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(self.tr('Menu hide timeout'), binder)
         self.__component['autoHideNodeMenuTimeout'] = binder
 
-    def _createShowNodeMenuAnimationDuration(self):
+    def _createshowNodeMenuAnimationDuration(self):
         '''
         Create showNodeMenuAnimationDuration components
         '''
@@ -585,7 +685,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(self.tr('Menu appearance duration'), binder)
         self.__component['showNodeMenuAnimationDuration'] = binder
 
-    def _createHideNodeMenuAnimationDuration(self):
+    def _createhideNodeMenuAnimationDuration(self):
         '''
         Create hideNodeMenuAnimationDuration components
         '''
@@ -594,7 +694,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(self.tr('Menu disappearance duration'), binder)
         self.__component['hideNodeMenuAnimationDuration'] = binder
 
-    def _createDefaultFont(self):
+    def _createdefaultFont(self):
         '''
         Create defaultFont components
         '''
@@ -605,7 +705,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(self.tr('Text font'), layout)
         self.__component['defaultFont'] = binder
 
-    def _createDefaultLabelFont(self):
+    def _createdefaultLabelFont(self):
         '''
         Create defaultLabelFont components
         '''
@@ -617,7 +717,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(self.tr('Label font'), layout)
         self.__component['defaultLabelFont'] = binder
 
-    def _createDefaultNodeTextColor(self):
+    def _createdefaultNodeTextColor(self):
         '''
         Create defaultNodeTextColor components
         '''
@@ -631,7 +731,7 @@ class SettingsCreator(QObject):
         binderButton.connect(SIGNAL('clicked()'), binder.open)
         self.__component['defaultNodeTextColor'] = binder
 
-    def _createDefaultNodeBackgroundColor(self):
+    def _createdefaultNodeBackgroundColor(self):
         '''
         Create defaultNodeBackgroundColor components
         '''
@@ -645,7 +745,7 @@ class SettingsCreator(QObject):
         binderButton.connect(SIGNAL('clicked()'), binder.open)
         self.__component['defaultNodeBackgroundColor'] = binder
 
-    def _createDefaultEdgeColor(self):
+    def _createdefaultEdgeColor(self):
         '''
         Create defaultEdgeColor components
         '''
@@ -659,7 +759,7 @@ class SettingsCreator(QObject):
         binderButton.connect(SIGNAL('clicked()'), binder.open)
         self.__component['defaultEdgeColor'] = binder
 
-    def _createDefaultLabelColor(self):
+    def _createdefaultLabelColor(self):
         '''
         Create defaultLabelColor components
         '''
@@ -673,7 +773,7 @@ class SettingsCreator(QObject):
         binderButton.connect(SIGNAL('clicked()'), binder.open)
         self.__component['defaultLabelColor'] = binder
 
-    def _createDefaultEdgeWidth(self):
+    def _createdefaultEdgeWidth(self):
         '''
         Create defaultEdgeWidth components
         '''
@@ -683,7 +783,7 @@ class SettingsCreator(QObject):
         self.__mainLayout.addRow(self.tr('Edge width'), binder)
         self.__component['defaultEdgeWidth'] = binder
 
-    def _createDefaultSelectedNodeBackgroundColor(self):
+    def _createdefaultSelectedNodeBackgroundColor(self):
         '''
         Create defaultSelectedNodeBackgroundColor component
         '''
@@ -697,6 +797,34 @@ class SettingsCreator(QObject):
         binderButton.connect(SIGNAL('clicked()'), binder.open)
         self.__component['defaultSelectedNodeBackgroundColor'] = binder
 
+    def _createautosaveEnabled(self):
+        '''
+        Create autosaveEnabled component
+        '''
+        binder = CheckBoxBinder('autosaveEnabled', settings.get('autosaveEnabled'))
+        binder.setText(self.tr('Enable autosave of mind map'))
+        self.__mainLayout.addRow(binder)
+        self.__component['autosaveEnabled'] = binder
+
+    def _createautosaveInterval(self):
+        '''
+        Create autosaveInterval component
+        '''
+        binder = SpinBoxBinder('autosaveInterval',
+                settings.get('autosaveInterval'), 1, 600)
+        binder.setSuffix(self.tr(' sec'))
+        self.__mainLayout.addRow(self.tr('Autosave interval'), binder)
+        self.__component['autosaveInterval'] = binder
+
+    def _createuseExtendedMenu(self):
+        '''
+        Create useExtendedMenu component
+        '''
+        binder = CheckBoxBinder('useExtendedMenu', settings.get('useExtendedMenu'))
+        binder.setText(self.tr('Use extended menu'))
+        self.__mainLayout.addRow(binder)
+        self.__component['useExtendedMenu'] = binder
+
 
 class ComponentBinder(QObject):
     '''
@@ -797,7 +925,7 @@ class SpinBoxBinder(QSpinBox, ComponentBinder):
         self.setMaximum(max)
         self.setValue(value)
         self._value = value
-        self.valueChanged.connect(self._setValue)
+        self.valueChanged[int].connect(self._setValue)
 
 
 class CheckBoxBinder(QCheckBox, ComponentBinder):
@@ -934,6 +1062,7 @@ class FontBinder(QWidget, ComponentBinder):
         self.__size = font.pointSize()
         self.__sizeCombo = QComboBox()
         self.__sizeCombo.addItem(str(self.__size))
+        self.__sizeCombo.setEditable(True)
         self.__fontCombo = QFontComboBox()
         self.__fontCombo.setCurrentFont(font)
         self._setFont(font)
@@ -980,8 +1109,10 @@ class FontBinder(QWidget, ComponentBinder):
         @type font: QFont
         '''
         self.__sizeCombo.clear()
-        self.__sizeCombo.addItems([str(value) for value in QFontDatabase().\
-                                   pointSizes(font.family())])
+        sizes = [str(value) for value in QFontDatabase().pointSizes(font.family())]
+        if str(self.__size) not in sizes:
+            sizes.append(str(self.__size))
+        self.__sizeCombo.addItems(sizes)
 
 
 class FullScreenExitButton(QToolButton):
@@ -1027,3 +1158,55 @@ class FullScreenExitButton(QToolButton):
             if self.isVisible():
                 self.move(parent.width() - self.width(), parent.height() - self.height())
         return QToolButton.eventFilter(self, obj, event)
+
+
+class AffiliationDelegate(QStyledItemDelegate):
+    '''
+    Delegate for affiliation combo box
+    @author: Oleg Kandaurov
+    '''
+
+    readable('supportedAffiliations')
+
+    def __init__(self, parent = None):
+        '''
+        Constructor
+        '''
+        QStyledItemDelegate.__init__(self, parent)
+        self.__supportedAffiliations = {'owner': self.tr('Owner'),
+                'member' : self.tr('Member'),
+                'outcast': self.tr('Outcast'),
+                'publisher': self.tr('Publisher')}
+
+    def createEditor(self, parent, option, index):
+        '''
+        Returns the widget used to edit the item specified by index for editing.
+        The parent widget and style option are used to control how the editor widget appears.
+        '''
+        editor = QComboBox(parent)
+        for data, name in self.__supportedAffiliations.iteritems():
+            editor.addItem(name, data)
+        return editor
+
+    def setEditorData (self, editor, index):
+        '''
+        Sets the data to be displayed and edited by the editor
+        from the data model item specified by the model index.
+        '''
+        value = index.model().data(index, Qt.EditRole)
+        editor.setCurrentIndex(editor.findData(self.__supportedAffiliations[value]))
+
+    def setModelData (self, editor, model, index):
+        '''
+        Gets data from the editor widget and stores
+        it in the specified model at the item index.
+        '''
+        value = editor.itemData(editor.currentIndex())
+        model.setData(index, value, Qt.EditRole)
+
+    def updateEditorGeometry (self, editor, option, index):
+        '''
+        Updates the editor for the item specified
+        by index according to the style option given.
+        '''
+        editor.setGeometry(option.rect)
