@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -86,16 +86,20 @@ class MaemoEditNodeDialog(MaemoDialogMixin, EditNodeDialog):
         self._clearFormatAction.setIcon(QIcon(resource.getImage('node_editor_clear')))
         toolbar.addAction(self._clearFormatAction)
         #create edit widget
+        scalingWidget = ScalingWidget(self._editor, 2.0)
+        self._editor.setFrameShape(QFrame.Box)
         editWidget = QWidget()
         editLayout = QGridLayout()
         editWidget.setLayout(editLayout)
         editLayout.addWidget(toolbar, 0, 0, 1, 6)
-        editLayout.addWidget(self._editor, 1, 0, 1, 6)
+        editLayout.addWidget(scalingWidget, 1, 0, 1, 6)
         #create html widget
+        scalingWidget = ScalingWidget(self._htmlEditor, 1.5)
+        self._htmlEditor.setFrameShape(QFrame.Box)
         htmlWidget = QWidget()
         htmlLayout = QGridLayout()
         htmlWidget.setLayout(htmlLayout)
-        htmlLayout.addWidget(self._htmlEditor, 0, 0)
+        htmlLayout.addWidget(scalingWidget, 0, 0)
         #create stacked widget
         self.__stackedWidget = QStackedWidget()
         self.__textIndex = self.__stackedWidget.addWidget(editWidget)
@@ -108,6 +112,7 @@ class MaemoEditNodeDialog(MaemoDialogMixin, EditNodeDialog):
         mainLayout.addWidget(self.__showText, 2, 0)
         mainLayout.addWidget(self.__showHtml, 2, 1)
         mainLayout.addWidget(self._buttonBox, 2, 2, 1, 4)
+        self._editor.setFocus()
         self.setLayout(mainLayout)
         self.__showHtml.connect(SIGNAL('toggled(bool)'), adaptCallable(self._changedMode))
         self._textHtmlAction.connect(SIGNAL('toggled(bool)'),
@@ -171,14 +176,17 @@ class MaemoEditLabelDialog(MaemoDialogMixin, EditLabelDialog):
         secondaryLayout.addSpacing(20)
         secondaryLayout.addWidget(self._colorParent)
         secondaryLayout.addWidget(self._colorButton)
+        scalingWidget = ScalingWidget(self._editor, 2.0)
+        self._editor.setFrameShape(QFrame.Box)
         mainLayout = QGridLayout()
         mainLayout.addWidget(toolbar, 0, 0, 1, 2)
         mainLayout.addWidget(self._comboFont, 0, 2, 1, 5)
         mainLayout.addWidget(self._comboSize, 0, 7, 1, 3)
-        mainLayout.addWidget(self._editor, 1, 0, 1, 10)
+        mainLayout.addWidget(scalingWidget, 1, 0, 1, 10)
         mainLayout.addLayout(secondaryLayout, 2, 0, 1, 10)
         mainLayout.addWidget(self._buttonBox, 3, 0, 1, 10)
         self.setLayout(mainLayout)
+        self._editor.setFocus()
         QWidget.setTabOrder(self._editor, self._buttonBox)
         QWidget.setTabOrder(self._buttonBox, self._comboFont)
         QWidget.setTabOrder(self._comboFont, self._comboSize)
@@ -241,7 +249,7 @@ class MaemoAboutDialog(MaemoDialogMixin, AboutDialog):
                             This application  is licensed under the terms <br>of the
                             <a href="http://www.gnu.org/licenses/gpl-3.0.html">
                             GNU General Public License v3</a>
-                            <p><br>Copyright &copy; 2010 HiveMind developers</center>
+                            <p><br>Copyright &copy; 2010-2011 HiveMind developers</center>
                             <ul style="list-style-type: square;margin: 0px 0px 0px 40px">
                             <li><a href="mailto:golovchenkoaa@gmail.com">Andrey Golovchenko</a>, core developer
                             <li><a href="mailto:vamonster@gmail.com">Andrey Vasilev</a>, developer/designer
@@ -252,6 +260,7 @@ class MaemoAboutDialog(MaemoDialogMixin, AboutDialog):
                             <a href="http://fruct.org/">
                             Open Innovations Framework Program FRUCT</a>
                             <br><br>HiveMind logo by <a href="http://nicubunu.ro">Nicu Buculei</a> (nicubunu)
+                            <br>Notification icons by <a href="http://www.mentalrey.it/">Mentalrey</a>
                             <br><br><br><br><br><br><br><br><br>
                             </center>'''))
         self._about.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
@@ -352,8 +361,9 @@ class MaemoSettingsDialog(MaemoDialogMixin, SettingsDialog):
         @type parent: QWidget
         '''
         SettingsDialog.__init__(self, parent)
+        MaemoDialogMixin.__init__(self)
         tabWidget = QTabWidget()
-        self._general = SettingsCreator(['locale', 'confirmDelete'])
+        self._general = SettingsCreator(['locale', 'confirmDelete', 'useExtendedMenu'])
         generalTab = QWidget()
         generalTab.setLayout(self._general.getLayout())
         self._node = SettingsCreator(['defaultFont',
@@ -373,11 +383,15 @@ class MaemoSettingsDialog(MaemoDialogMixin, SettingsDialog):
                 'hideNodeMenuAnimationDuration'])
         animationTab = QWidget()
         animationTab.setLayout(self._animation.getLayout())
+        self._autosave = SettingsCreator(['autosaveEnabled', 'autosaveInterval'])
+        autosaveTab = QWidget()
+        autosaveTab.setLayout(self._autosave.getLayout())
         tabWidget.addTab(generalTab, self.tr('General'))
         tabWidget.addTab(nodeTab, self.tr('Node'))
         tabWidget.addTab(edgeTab, self.tr('Edge'))
         tabWidget.addTab(edgeLabelTab, self.tr('Edge Label'))
         tabWidget.addTab(animationTab, self.tr('Animation'))
+        tabWidget.addTab(autosaveTab, self.tr('Autosave'))
         mainLayout = QVBoxLayout()
         mainLayout.addWidget(tabWidget)
         mainLayout.addWidget(self._buttonBox)
@@ -392,6 +406,7 @@ class MaemoSettingsDialog(MaemoDialogMixin, SettingsDialog):
         self._newSettings.append(self._edge.retrieve())
         self._newSettings.append(self._edgeLabel.retrieve())
         self._newSettings.append(self._animation.retrieve())
+        self._newSettings.append(self._autosave.retrieve())
         SettingsDialog._accept(self)
 
 class MaemoXMPPCredentialsDialog(MaemoDialogMixin, XMPPCredentialsDialog):
@@ -401,3 +416,97 @@ class MaemoXMPPCredentialsDialog(MaemoDialogMixin, XMPPCredentialsDialog):
     '''
     def __init__(self, client = False, parent = None):
         XMPPCredentialsDialog.__init__(self, client, parent)
+        MaemoDialogMixin.__init__(self)
+
+
+class MaemoAffiliationDialog(MaemoDialogMixin, AffiliationDialog):
+    '''
+    Affiliation dialog for Maemo platform
+    @author: Oleg Kandaurov
+    '''
+    def __init__(self, parent = None):
+        AffiliationDialog.__init__(self, parent)
+        MaemoDialogMixin.__init__(self)
+
+
+class MaemoExtendedMenuDialog(QDialog):
+    '''
+    Extended menu dialog for Maemo platform
+    @author: Alexander Kulikov
+    '''
+    def __init__(self, actionBag, freezeTime = 0, parent = None):
+        '''
+        @type actionBag: ActionBag
+        @param freezeTime: Time interval during which dialog can not be rejected
+        @type freezeTime: int
+        @type parent: None
+        '''
+        QDialog.__init__(self, parent)
+        self.setWindowTitle(self.tr('Extended menu'))
+        self.__freezed = freezeTime != 0
+        QTimer.singleShot(freezeTime, self._unfreeze)
+        self._createComponents(actionBag)
+
+    def _createComponents(self, actionBag):
+        '''
+        Create dialog widgets
+        @type actionBag: ActionBag
+        '''
+        layout = QGridLayout()
+        font = QFont()
+        font.setPixelSize(28)
+        singleClickLabel = QLabel(self.tr('Single-click on the node to show the main menu:'))
+        singleClickLabel.setFont(font)
+        doubleClickLabel = QLabel(self.tr('Double-click on the node to show the additional menu:'))
+        doubleClickLabel.setFont(font)
+        layout.addWidget(singleClickLabel, 0, 0, 1, 4, Qt.AlignCenter)
+        layout.addWidget(doubleClickLabel, 3, 0, 1, 4, Qt.AlignCenter)
+        layout.addWidget(self._createActionWidget(actionBag.addNodeAction), 1, 0,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Add node')), 2, 0, Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.editNodeAction), 1, 1,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Edit node')), 2, 1, Qt.AlignHCenter |
+                Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.removeNodeAction), 1, 2,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Remove node')), 2, 2, Qt.AlignHCenter |
+                Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.foldNodeAction), 1, 3,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('<center>Fold/unfold<br>node subtree</center>')),
+                2, 3, Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.editNodeIconsAction), 4, 0,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Select icons')), 5, 0, Qt.AlignHCenter |
+                Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.editEdgeAction), 4, 1,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Edit edge')), 5, 1, Qt.AlignHCenter |
+                Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.enterTransferModeAction), 4, 2,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Move node')), 5, 2, Qt.AlignHCenter |
+                Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.editLabelAction), 4, 3,
+                Qt.AlignCenter)
+        layout.addWidget(QLabel(self.tr('Edit edge label')), 5, 3, Qt.AlignHCenter |
+                Qt.AlignTop)
+        self.setLayout(layout)
+
+    def _unfreeze(self):
+        '''Unfreeze reject functionality'''
+        self.__freezed = False
+
+    def reject(self):
+        if self.__freezed == False:
+            QDialog.reject(self)
+
+    def _createActionWidget(self, action):
+        '''
+        Create action widget
+        @type action: QAction
+        '''
+        button = ActionImageButton(action)
+        button.connect(SIGNAL('clicked()'), self.accept)
+        return button
