@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -30,36 +30,27 @@ from hivemind.gui_delegates import *
 
 class MainView(QGraphicsView):
     '''
-    Main view of the application displaying mind map
+    Main view of the application displaying mind map.
     
     @author: Andrew Vasilev
     '''
 
-    __MIN_DRAG_PATH = 20
-    '''Minimum drag path length'''
+    readable('draggedPath')
 
-    __MIN_DRAG_DURATION = 200
-    '''Minimum drag duration'''
-
-    def __init__(self, controller, *values):
-        QGraphicsView.__init__(self, *values)
+    def __init__(self, scene, actionBag):
+        QGraphicsView.__init__(self, scene)
+        self.setRenderHints(QPainter.Antialiasing)
         self.setDragMode(QGraphicsView.ScrollHandDrag)
-        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.setResizeAnchor(QGraphicsView.AnchorViewCenter)
-        self.__controller = controller
+        palette = self.palette()
+        palette.setColor(QPalette.Base, Qt.white)
+        self.setPalette(palette)
         self.__draggedPath = 0
-        self.__hideMenuTimer = QTimer(self)
-        self.__hideMenuTimer.setInterval(MainView.__MIN_DRAG_DURATION)
-        self.__hideMenuTimer.setSingleShot(True)
-        self.__hideContextMenuTimer = QTimer(self)
-        self.__hideContextMenuTimer.setInterval(MainView.__MIN_DRAG_DURATION)
-        self.__hideContextMenuTimer.setSingleShot(True)
-
+        self.__actionBag = actionBag
         self.__zoomLevel = 0
-        # FIXME: Move the following into controller
-        controller.connect(self, SIGNAL('itemClicked'),
-                adaptCallable(controller.onItemClicked))
+        self.__actionBag.zoomInAction.connect(SIGNAL('triggered()'), self._zoomIn)
+        self.__actionBag.zoomOutAction.connect(SIGNAL('triggered()'), self._zoomOut)
+        self.connect(scene, SIGNAL('mapChanged'), self._resetView)
 
     def scrollContentsBy(self, dx, dy):
         '''
@@ -80,8 +71,6 @@ class MainView(QGraphicsView):
         QGraphicsView.mousePressEvent(self, event)
         self.__draggedPath = 0
         self.__startPoint = event.pos()
-        self.__hideMenuTimer.start()
-        self.__hideContextMenuTimer.start()
 
     def mouseReleaseEvent(self, event):
         '''
@@ -92,16 +81,6 @@ class MainView(QGraphicsView):
         '''
         QGraphicsView.mouseReleaseEvent(self, event)
         self.__draggedPath += self._length(self.__startPoint, event.pos())
-        if self.__draggedPath < MainView.__MIN_DRAG_PATH and \
-            (self.__hideMenuTimer.isActive() or self.__hideContextMenuTimer.isActive()):
-            item = self.scene().itemAt(self.mapToScene(event.pos()))
-            parent, childLocation = self.scene().parentAt(self.mapToScene(event.pos()))
-            if parent is not None:
-                self.emit(SIGNAL('itemClicked'), event.button() == Qt.RightButton,
-                        item, parent, childLocation)
-            if item is None:
-                self.emit(SIGNAL('mapClicked'))
-
 
     def _length(self, point1, point2):
         '''
@@ -121,36 +100,51 @@ class MainView(QGraphicsView):
     __ZOOM_OUT_COEFFICIENT = 0.8
     '''View scale coefficient for zoom out action'''
 
-    __MAX_ZOOM_LEVEL = 7
+    __MAX_ZOOM_LEVEL = 8
     '''Maximum value for zoom level'''
 
-    __MIN_ZOOM_LEVEL = -7
+    __MIN_ZOOM_LEVEL = -8
     '''Minimum value for zoom level'''
 
-    def zoomIn(self):
+    def _zoom(self, scaleFactor):
+        '''
+        Change the scale factor of the mind map.
+        @parame scaleFactor: the scale of the map to apply (__ZOOM_IN_COEFFICIENT or
+        _ZOOM_OUT_COEFFICIENT)
+        @type scaleFactor: double
+        '''
+        self.scale(scaleFactor, scaleFactor)
+        self.__zoomLevel += 1 if scaleFactor > 1 else - 1
+        self._updateZoomActionsState()
+
+    def _updateZoomActionsState(self):
+        '''Update the state of the actions according to
+        the zoom level'''
+        if self.__zoomLevel == MainView.__MAX_ZOOM_LEVEL:
+            self.__actionBag.zoomInAction.setEnabled(False)
+        elif self.__zoomLevel == MainView.__MIN_ZOOM_LEVEL:
+            self.__actionBag.zoomOutAction.setEnabled(False)
+        else:
+            self.__actionBag.zoomInAction.setEnabled(True)
+            self.__actionBag.zoomOutAction.setEnabled(True)
+
+    def _zoomIn(self):
         '''Zoom in scene view'''
-        if self.__zoomLevel > MainView.__MAX_ZOOM_LEVEL:
-            return
-        self.__zoomLevel += 1
-        scaleFactor = MainView.__ZOOM_IN_COEFFICIENT
-        self.scale(scaleFactor, scaleFactor)
+        self._zoom(MainView.__ZOOM_IN_COEFFICIENT)
 
-    def zoomOut(self):
+    def _zoomOut(self):
         '''Zoom out scene view'''
-        if self.__zoomLevel < MainView.__MIN_ZOOM_LEVEL:
-            return
-        self.__zoomLevel -= 1
-        scaleFactor = MainView.__ZOOM_OUT_COEFFICIENT
-        self.scale(scaleFactor, scaleFactor)
+        self._zoom(MainView.__ZOOM_OUT_COEFFICIENT)
 
-    def resetView(self):
+    def _resetView(self):
         '''
         Reset zoom transformations. Scroll the contents of
-        the viewport to ensure that the item is centered in the view.
+        the viewport to ensure that the root item is centered in the view.
         '''
-        self.resetMatrix()
-        self.resetTransform ()
+        self.resetTransform()
         self.scale(1.44, 1.44)
+        self.__zoomLevel = 0
+        self._updateZoomActionsState()
         self.centerOn(self.scene().rootDelegate)
 
 
@@ -161,19 +155,15 @@ class MaemoNodeMenu(QObject):
     @author: Andrew Vasilev, Alexander Kulikov
     '''
 
-    def __init__(self, parent, controller):
+    def __init__(self, parent):
         '''
-        @param controller: the controller
-        @type controller: MainWindowController
         @param parent: the main scene view to place menu on top of
         @type parent: MainView
         '''
         QObject.__init__(self, parent)
         self._mapView = parent
-        self._controller = controller
         self._menuButtons = {}
         self.connect(parent, SIGNAL('mapClicked'), self.hideMenu)
-
         self.__hideMenuTimer = QTimer(self)
         self.__hideMenuTimer.setInterval(settings.get('autoHideNodeMenuTimeout'))
         self.__hideMenuTimer.setSingleShot(True)
@@ -188,8 +178,8 @@ class MaemoNodeMenu(QObject):
         @type action: QAction
         @type iconName: str
         '''
-        button = ImageButton(self._mapView, iconName)
-        button.setCheckable(action.isCheckable())
+        action.setIcon(resource.getIcon(iconName))
+        button = ActionImageButton(action, self._mapView)
         self._connectButtonToAction(button, action)
 
     def _connectButtonToAction(self, button, action):
@@ -200,9 +190,7 @@ class MaemoNodeMenu(QObject):
         '''
         self._menuButtons[action] = button
         button.hide()
-        action.connect(SIGNAL('toggled(bool)'), button, SLOT('setChecked(bool)'))
         button.connect(SIGNAL('clicked()'), self.hideMenu)
-        button.connect(SIGNAL('clicked()'), action.trigger)
 
     def _restartHideMenuTimer(self):
         '''Restart hide menu timer'''
@@ -420,51 +408,52 @@ class AbstractMainWindow(SearchingTrMixin, QMainWindow):
     '''
     Abstract main window class
 
+    Subclasses of this class must implement self._mindMapView property
+
     @author: Alexander Kulikov
     '''
-    def __init__(self, controller, mindMapScene):
+
+    def _getMindMapView(self):
+        return self._mindMapView
+
+    mindMapView = property(_getMindMapView, None, None, 'mindmap view')
+
+    def __init__(self, actionBag, mindMapScene):
         '''Create new window'''
         QMainWindow.__init__(self)
-        self._mindMapView = MainView(controller, mindMapScene)
-        self._mindMapView.setRenderHints(QPainter.Antialiasing)
-        palette = self._mindMapView.palette()
-        palette.setColor(QPalette.Base, Qt.white)
-        self._mindMapView.setPalette(palette)
-        self.setCentralWidget(self._mindMapView)
-        self.__controller = controller
-        self._initActions(controller)
+        self._actionBag = actionBag
+        self._initNavigateActions(mindMapScene)
+        self._initActions()
 
-    def _initActions(self, controller):
+    def _initNavigateActions(self, mindMapScene):
+        '''Appends the navigate actions to mainView's list of actions'''
+        self.addAction(mindMapScene.selectLeftNodeAction)
+        self.addAction(mindMapScene.selectRightNodeAction)
+        self.addAction(mindMapScene.selectUpNodeAction)
+        self.addAction(mindMapScene.selectDownNodeAction)
+        self.addAction(mindMapScene.selectRootNodeAction)
+
+    def _initActions(self):
         '''Appends the actions to mainView's list of actions'''
-        self._mindMapView.addAction(controller.editEdgeAction)
-        self._mindMapView.addAction(controller.editLabelAction)
-        self._mindMapView.addAction(controller.editNodeAction)
-        self._mindMapView.addAction(controller.editNodeIconsAction)
-        self._mindMapView.addAction(controller.foldNodeAction)
-        self._mindMapView.addAction(controller.addNodeAction)
-        self._mindMapView.addAction(controller.removeNodeAction)
-        self._mindMapView.addAction(controller.selectLeftNodeAction)
-        self._mindMapView.addAction(controller.selectRightNodeAction)
-        self._mindMapView.addAction(controller.selectUpNodeAction)
-        self._mindMapView.addAction(controller.selectDownNodeAction)
-        self._mindMapView.addAction(controller.selectRootNodeAction)
-        self._mindMapView.addAction(controller.addAdjacentNodeAction)
-        self._mindMapView.addAction(controller.zoomInAction)
-        self._mindMapView.addAction(controller.zoomOutAction)
-        controller.zoomInAction.connect(SIGNAL('triggered()'),
-            self._mindMapView.zoomIn)
-        controller.zoomOutAction.connect(SIGNAL('triggered()'),
-            self._mindMapView.zoomOut)
+        self.addAction(self._actionBag.editEdgeAction)
+        self.addAction(self._actionBag.editLabelAction)
+        self.addAction(self._actionBag.editNodeAction)
+        self.addAction(self._actionBag.editNodeIconsAction)
+        self.addAction(self._actionBag.foldNodeAction)
+        self.addAction(self._actionBag.addNodeAction)
+        self.addAction(self._actionBag.removeNodeAction)
+        self.addAction(self._actionBag.enterTransferModeAction)
+        self.addAction(self._actionBag.cancelTransferModeAction)
+        self.addAction(self._actionBag.putNodeAction)
+        self.addAction(self._actionBag.putNodeBelowAction)
+        self.addAction(self._actionBag.putNodeAboveAction)
 
-    def resetView(self):
-        '''
-        Resets the view transformation
-        '''
-        self._mindMapView.resetView()
+        self.addAction(self._actionBag.addSiblingNodeBelowAction)
+        self.addAction(self._actionBag.addSiblingNodeAboveAction)
+        self.addAction(self._actionBag.zoomInAction)
+        self.addAction(self._actionBag.zoomOutAction)
 
     def closeEvent(self, event):
         '''Close event handler'''
-        if not self.__controller.prohibitChangeMapAction():
-            event.ignore()
-        else:
-            self.__controller.stopNetworkCommunication()
+        self.emit(SIGNAL('closeEvent'))
+        event.ignore()
