@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in @author or @authors annotations at the beginning of each
@@ -56,6 +56,8 @@ def _parseNode(root, parent = None):
     if nodePosition is not None: storeAttrs['left'] = nodePosition == 'left'
     nodeColor = attributes.pop('color', None)
     if nodeColor is not None: storeAttrs['color'] = nodeColor
+    backgroundColor = attributes.pop('background_color', None)
+    if backgroundColor is not None: storeAttrs['background_color'] = backgroundColor
     node = parent.createChildPrototype(**attributes) if parent else Node(**attributes)
     icons = []
     # parse child node attributes
@@ -495,6 +497,18 @@ def _fillStateModificationCommandElement(command, commandElement):
     stateAfter.attrib.update(_toString(command.stateAfter, toUpperCase = False))
     commandElement.append(stateAfter)
 
+def _fillstrElement(command, commandElement):
+    '''
+    Add required content to string XML element
+
+    @param command: command object
+    @type command: str
+
+    @param commandElement: XML element for processing
+    @type commandElement: lxml Element
+    '''
+    commandElement.append(xml.Element('action', type = command))
+
 def _commandToXml(command):
     '''
     Create lxml Element from command object
@@ -530,7 +544,7 @@ def serializeCommand(command):
 def serializeCommandStack(stack):
     '''
     Serialize the stack of commands to XML in the following format:
-        <commandStack>
+        <commandStack index="...">
             <command type="..."></command>
             ...
             <command type="..."></command>
@@ -543,6 +557,7 @@ def serializeCommandStack(stack):
     @rtype: unicode
     '''
     stackElement = xml.Element('commandStack')
+    stackElement.set('index', str(stack.index()))
     for index in xrange(stack.count()):
         stackElement.append(_commandToXml(stack.command(index)))
     return xml.tostring(stackElement, encoding = 'UTF-8')
@@ -599,6 +614,15 @@ def _deserializeStateModificationCommand(commandElement, mindMap):
     stateBefore = node.getAttributes(*stateAfter.keys())
     return StateModificationCommand(node, stateBefore, stateAfter)
 
+def _deserializestr(commandElement, mindMap):
+    '''
+    Deserialize string command from XML string
+
+    @rtype: str
+    '''
+    nodeElement = commandElement[0]
+    return nodeElement.get('type')
+
 def _xmlToCommand(commandElement, mindMap):
     '''
     Create command object from lxml Element
@@ -647,6 +671,7 @@ def deserializeCommandStack(xmlString, mindMap):
     stackElement = xml.fromstring(xmlString)
     for commandElement in stackElement:
         commandStack.push(_xmlToCommand(commandElement, mindMap))
+    commandStack.setIndex(int(stackElement.get('index')))
     return commandStack
 
 def storeMindMap(fileName, mindMap):
