@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in  or  annotations at the beginning of each
@@ -59,6 +59,29 @@ def getImage(name):
         QPixmapCache.insert(fullname, image)
     return image
 
+__NORMAL_STATE_IMAGE_SUFFIX = '_normal'
+'''Suffix for image to be displayed in normal mode'''
+
+__SELECTED_STATE_IMAGE_SUFFIX = '_selected'
+'''Suffix for image to be displayed in selected mode'''
+
+__DISABLED_STATE_IMAGE_SUFFIX = '_disabled'
+'''Suffix for imagee to be displayed in disabled mode'''
+
+def getIcon(name):
+    '''
+    @param name: base name of the pictures representig icon
+    @type name: str
+    
+    @return: new icon containg normal, selected and disabled states image
+    @rtype: QIcon
+    '''
+    icon = QIcon()
+    icon.addPixmap(getImage(name + __NORMAL_STATE_IMAGE_SUFFIX), QIcon.Normal, QIcon.Off)
+    icon.addPixmap(getImage(name + __SELECTED_STATE_IMAGE_SUFFIX), QIcon.Selected, QIcon.Off)
+    icon.addPixmap(getImage(name + __DISABLED_STATE_IMAGE_SUFFIX), QIcon.Disabled, QIcon.Off)
+    return icon
+
 def _loadIcons():
     '''
     Load icons to the hash table using a list of
