@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # HiveMind - Distributed mind map editor for Maemo 5 platform
-# Copyright (C) 2010 HiveMind developers
+# Copyright (C) 2010-2011 HiveMind developers
 #
 # HiveMind is the legal property of its developers, whose names are
 # noticed in  or  annotations at the beginning of each
@@ -32,8 +32,9 @@ import os
 
 import hivemind.__dirs__ as __dirs__
 
-__INT_PARAMETERS = ('defaultEdgeWidth', 'autoHideNodeMenuTimeout', 'showNodeMenuAnimationDuration',
-                    'hideNodeMenuAnimationDuration')
+__INT_PARAMETERS = ('defaultEdgeWidth', 'autoHideNodeMenuTimeout',
+        'showNodeMenuAnimationDuration', 'hideNodeMenuAnimationDuration',
+        'xmppKeepAliveInterval', 'xmppKeepAliveCount', 'autosaveInterval')
 '''
 Integer parameters names
 '''
@@ -55,7 +56,8 @@ __COLOR_PARAMETERS = ('defaultNodeTextColor', 'defaultNodeBackgroundColor',
 Font parameters names
 '''
 
-__BOOLEAN_PARAMETERS = ('confirmDelete',)
+__BOOLEAN_PARAMETERS = ('confirmDelete', 'autosaveEnabled', 'readOnly', 'firstClick',
+                        'useExtendedMenu')
 '''
 Boolean parameters names
 '''
@@ -85,7 +87,9 @@ def get(parameter):
     elif parameter in __COLOR_PARAMETERS:
         return QColor(value)
     elif parameter in __BOOLEAN_PARAMETERS:
-        return bool(value)
+        if value is True or value is False:
+            return value
+        return {'true': True, 'false': False}.get(value.lower())
     else:
         return value
 
